with Ada.Directories;
with Ada.Text_IO;
with Casendra.Comments;
with Gnatcoll.Json;
with Casendra.Directories;
with Casendra.Text_Utils;
package body Casendra.Cases is
   function Attachments (Self : in Case_T) return Casendra.Attachments.Attachments_T is (Self.Attachments);
   function Is_Closed (Self : in Case_T) return Boolean is (Self.Is_Closed);
   function Id (Self : in Case_T) return String is (To_String(Self.Id));

   procedure Init (Self : out Case_T; Case_Id : in String) is
   begin
      Casendra.Strata.Connect (Self.Connection);
      Self.Id := To_Unbounded_String (Case_Id);
      declare
         use Gnatcoll.Json;
         Response : constant String := Casendra.Strata.Get_Case_JSON (Case_Id, Self.Connection);
         Response_Json : Gnatcoll.Json.Json_Value;
      begin
         Response_Json := Gnatcoll.Json.Read (Response, Filename => "");
         Self.Is_Closed := (if Has_Field (Response_Json, "closed") then Get (Response_Json, "closed") else True);
         Self.Summary :=  (if Has_Field (Response_Json, "summary") then To_Unbounded_String( String'(Get (Response_Json, "summary"))) else Null_Unbounded_String);
         Self.Description :=  (if Has_Field (Response_Json, "description") then To_Unbounded_String( String'(Get (Response_Json, "description"))) else Null_Unbounded_String);
      end;

      -- TODO
      -- Retrieve one single json here? 
      Self.Attachments := Casendra.Attachments.Init (Case_Id, Self.Connection);
      Self.Comments := Casendra.Comments.Init (Case_Id, Self.Connection);
   end Init;
   
   procedure Download_Attachment (Self     : in out Case_T;
                                  Attachments : in              Casendra.Attachments.Attachments_T;
                                  Dir      : in String := "/tmp/";
                                  Callback   : in     Casendra.Strata.Progress_Callback_T := Casendra.Strata.Null_Callback)
   is

      Subdir : constant String := Dir & To_String (Self.Id) & "/attachments/";
   begin

      if not Ada.Directories.Exists (Subdir) then
         Ada.Directories.Create_Path (Subdir);
         Ada.Directories.Create_Path (Subdir & "/.by-uuid/");
      end if;
      Casendra.Attachments.Download (Attachments, Self.Connection, Subdir, Callback);
   exception
      when Ada.Directories.Use_Error =>
         pragma Debug (Ada.Text_IO.Put_Line ("Unable to create directory " & Subdir));
         null;
   end Download_Attachment;
   
   procedure Save_History (Self : in Case_T; Dir  : in String := "/tmp/") is
      Subdir                    : constant String := Dir & To_String (Self.Id);
      Filename                  : constant String := SubDir & "/comments.org";
      File                      : Ada.Text_Io.File_Type;
   begin
      if not Ada.Directories.Exists (Subdir) then
      Try :
         begin
            Ada.Directories.Create_Path (Subdir);
         exception
            when Ada.Text_IO.Use_Error =>
               raise;
         end Try;
      end if;
      
      if Ada.Directories.Exists (Filename) then
         -- Re-read comments
         Ada.Directories.Delete_File (Filename);
      end if;
      Ada.Text_IO.Create (File, Ada.Text_IO.Out_File, Filename);
      -- Ada.Text_IO.Set_Line_Length (File, 100);
      pragma Debug (Ada.Text_IO.Put_Line ("Saving comments: " & Filename));
      Ada.Text_Io.Put_Line (File, To_String ("* " & Casendra.Text_Utils.Escape_Org_Characters (Self.Summary)));
      Ada.Text_Io.Put_Line (File, To_String ("** Description:" & Ascii.Lf & Casendra.Text_Utils.Escape_Org_Characters (Self.Description)));
      Ada.Text_Io.Put_Line (File, To_String ("** Owner:" & Self.Owner));
      Ada.Text_Io.Put_Line (File, "** Comments");

      Casendra.Comments.Save_All (Self.Comments, File);
      if Ada.Text_IO.Is_Open (File) then
         Ada.Text_IO.Close (File);
      end if;

   exception
      when Ada.Text_IO.Use_Error | Ada.Text_IO.Mode_Error =>
         if Ada.Text_IO.Is_Open (File) then
            Ada.Text_IO.Close (File);
         end if;
         raise;
      when others =>
         pragma Debug (Ada.Text_Io.Put_Line ("Failed to save history"));
         raise;

   end Save_History;
   
   procedure Delete_Data (Self : in Case_T) 
   is
      Dir : constant String := Config_FIle.Get_String (Casendra.Config,
                                                       "downloader.directory",
                                                       False, 
                                                       "/tmp/folder/");

      Subdir : constant String := Dir & To_String (Self.Id);
   begin
      Casendra.Directories.Delete_Tree (Subdir);
   exception
      when Ada.Directories.Use_Error =>
         Ada.Text_Io.Put_Line ("Failed to delete directory" & Subdir & ". Probably because of permissions. Please delete it manually with:");
         Ada.Text_Io.Put_Line ("sudo rm -rf " & Subdir);
   end Delete_Data;
   
   function Get_All_Attachments (Self : in Case_T) return Casendra.Attachments.Attachments_T is (Self.Attachments);
   
   procedure Delete_Db (Self : in Case_T) is
   begin
      raise Program_Error with "Not implemented";
   end Delete_Db;
   
end Casendra.Cases;
