with Ada.Text_Io;
with Casendra.Text_Utils;
package body Casendra.Comment is
   
   function Format_Author_Date (Self : Comment_T) return Unbounded_String is
   
   begin
      return (if not Self.Public then "PRIVATE:" else "") & Self.Created_By & " at " & Casendra.Text_Utils.Date_To_String (Self.Date);
   end Format_Author_Date;
   
   function Init (Element_JSON : in GNATCOLL.JSON.JSON_Value) return Comment_T is
      Element : Comment_T;
      use GNATCOLL.JSON;
      use Casendra.Text_Utils;
   begin
      Element.Created_By := (if Has_Field (Element_Json, "created_by") then Get (Element_JSON, "created_by") else Null_Unbounded_String);
      Element.Date := (if Has_Field (Element_Json, "created_date") then Convert_Date_String (Get (Element_Json, "created_date")) else Epoch);
      Element.Text := (if Has_Field (Element_Json, "text") then Get (Element_JSON, "text") else Null_Unbounded_String);
      Element.Public := (if Has_Field (Element_JSON, "public") then Get (Element_JSON, "public") else False);
      Element.Raw_Text := Element_Json.Write;
      return Element;
      
   end Init;
   
   function To_Txt (Self : Comment_T) return String is
      Unbounded_Return : Unbounded_String := Null_Unbounded_String;
      use Casendra.Text_Utils;
   begin
      Append (Unbounded_Return, "*** " & Format_Author_Date (Self) & Ascii.LF);
      Append (Unbounded_Return, Ascii.Lf);
      Append (Unbounded_Return, Escape_Org_Characters (Self.Text));
      Append (Unbounded_Return, Ascii.Lf);
      return To_String (Unbounded_Return); -- FIXME
   end To_Txt;



end Casendra.Comment;
