with Ada.Strings.Fixed;
package body casendra.text_utils is 
   
   
   function String_To_Natural_Array (Input : in String; Delimeter : in String) return Natural_Array is
      Retval : Natural_Array (1 ..  Ada.Strings.Fixed.Count (Input, Delimeter) + 1);
      Next_Cursor : Natural := 0;
      Cursor : Natural := 0;
      function String_To_Natural (Input : in String) return Natural is
      begin
         return Natural'Value (Input);
      end String_To_Natural;
   begin
      if Retval'Length /= 1 then
         if Input = "" then
            raise PROGRAM_ERROR with "Empty string";
         end if;
         for Index in Retval'First .. Retval'Last - 1 loop
            Next_Cursor := Ada.Strings.Fixed.Index (Input (Cursor + 1 .. Input'Last), Delimeter);
            Retval (Index) := String_To_Natural (Input (Cursor + 1  .. Next_Cursor - 1));
            Cursor := Next_Cursor;
         end loop;
         -- Add last numeric value
         Retval (Retval'Last) := String_To_Natural (Input (Cursor + 1 .. Input'last));
      else
         Retval (1) := String_To_Natural (Input);
      end if;
      return Retval;
   end String_To_Natural_Array;
   
   function Convert_Date_String (Self : in Long_Integer) return Ada.Calendar.Time 
   is
      use Ada.Calendar;
   begin
      return Epoch + Duration(Self/1000);
   exception
      when Constraint_Error =>
         return Epoch;
   end Convert_Date_String;
   
   function Date_To_String (Date : Ada.Calendar.Time) return String is
   begin
      return "<" & Ada.Calendar.Formatting.Image (Date) & ">";
   end Date_To_String;
   
   
   function Escape_Org_Characters (Text : in Unbounded_String) return Unbounded_String is
      -- Recursively replace all "\n*" occurences with "\n *" to make org mode happy
      Pattern : constant String := Ascii.Lf & "*";
      Replace_With : constant String := Ascii.Lf & " *";
      Ind : constant Natural := Index (Text, Pattern);
   begin
      loop
         if Ind = 0 then
            return Text;
         end if;
         return Unbounded_Slice (
                                 Text, 
                                 Low => 1, 
                                 High => Ind - 1) & 
           Replace_With & 
           Escape_Org_Characters (Unbounded_Slice(Text,
                                                  Ind + Pattern'Length, 
                                                  Length(Text)));
      end loop;
      return Text;
   end Escape_Org_Characters;
   

   
end casendra.text_utils; 

